from dataclasses import dataclass
from typing import Optional, List

from wapi import Route, GET


@dataclass
class MemberDTO:
    id: int
    guild_id: int
    user_id: int
    name: str
    gender: str
    birthdate: str
    about: str
    lover: Optional[int]
    exp: int
    lvl: int
    up_exp: int
    wallet: int


@dataclass
class MetaResponse:
    message: str
    timestamp: str


@dataclass
class FetchMembersFromGuildRequest:
    meta: MetaResponse
    data: List[MemberDTO]


@Route("http://127.0.0.1:8080/guilds/{guild_id}/members")
class MembersService:

    @GET("/", FetchMembersFromGuildRequest, auto=True)
    def fetch_all_from_guild(
        self,
        *,
        guild_id: int,
        page: int = 1,
        size: int = 10,
        all: bool = False
    ) -> FetchMembersFromGuildRequest: pass

m = MembersService().fetch_all_from_guild(guild_id=1056553604246413313)

print(m)


from functools import wraps

from jsons import load, dump
from requests import Response, post

from wapi.static.T import T
from wapi.utils.get_path import get_path


def POST(
        path: str = "",
        _T: T = None,
) -> T | Response:
    def decorator(func):
        @wraps(func)
        def wrapper(self, **kwargs):
            headers = kwargs.pop('headers', None)
            data = dict()

            if body := dump(kwargs.pop('body', None)):
                data.update(body)

            url = get_path(self, func, path).format(**kwargs)
            response = post(url, json=data, headers=headers)

            if _T is None:
                return response
            return load(response.json(), _T)

        return wrapper

    return decorator



from functools import wraps

from jsons import load
from requests import Response, get

from wapi.static.T import T
from wapi.utils.get_path import get_path


def GET(
        path: str = "",
        _T: T = None,
        *,
        auto: bool = False,
        unpack: bool = False
) -> T | Response:
    def decorator(func):
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            headers = kwargs.pop("headers", None)
            p = get_path(self, func, path)
            url = p.format(*args, **kwargs)
            if auto:
                if unpack and (body := kwargs.pop('body', None)):
                    kwargs.update(body)

                url += "?" + "&".join([f'{k}={v}' for k, v in kwargs.items()])

            response = get(url, headers=headers)

            if _T is None:
                return response
            return load(response.json(), _T)

        return wrapper

    return decorator



from wapi.utils.merge_paths import merge_paths


def set_path(obj):
    for name, cls in vars(obj).items():
        if hasattr(cls, 'path'):
            cls.__class__.path = merge_paths(obj.path, cls.path)
            set_path(cls.__class__)
